/*
 *  Demonstrates how to use Read Buttons in conjunction with SoftRS485:
 *
 * If a button is pressed, a corresponding flag is set.
 * The main loop checks for the state of those flags and
 * sends a message on the RS485 bus whenever a flag is set.
 */

// include the library code:
#include <SoftRS485.h>

#define PROGRAM "RS485-Nano 2.1 / Sender 1.1"

#define BTN_INT 3 // button interrupt pin
#define Max485_RO 2 // read-output of Max485
#define Max485_RE 3 // not-read-enable of Max485
#define Max485_DE 8 // data enable of Max485
#define Max485_DI 9 // data input of Max485

#define TRESHOLD 100
#define ID 0
// 0=Test
// 1=Arbeitszimmer
// 2=Küche
// 3=Bad

//#define LOG_TO_SERIAL
#define SEND_485
#define DEBUG

boolean raw_states[8];
boolean states[8];
unsigned long capacitor[8];
unsigned long times[8];



void setup(){
  for (int i=0; i<8;i++) {
    capacitor[i] = 0;
    raw_states[i] = LOW;
    states[i] = LOW;
  }
  Serial.begin(115200);
  init485(Max485_RO,Max485_RE,Max485_DE,Max485_DI); // library initialization:
  pinMode(BTN_INT,INPUT_PULLUP);
  pinMode(14,INPUT_PULLUP);
  pinMode(15,INPUT_PULLUP);
  pinMode(16,INPUT_PULLUP);
  pinMode(17,INPUT_PULLUP);
  pinMode(18,INPUT_PULLUP);
  pinMode(19,INPUT_PULLUP);
  pinMode(20,INPUT_PULLUP);
  pinMode(21,INPUT_PULLUP);

  attachInterrupt(digitalPinToInterrupt(2),isr,CHANGE);
  // Print a message to the LCD.
  Serial.println(PROGRAM);
#ifdef LOG_TO_SERIAL
  Serial.println("14 15 16 17 18 19 20 21");
#endif
}

void isr(){
  raw_states[0] = !digitalRead(14);
  raw_states[1] = !digitalRead(15);
  raw_states[2] = !digitalRead(16);
  raw_states[3] = !digitalRead(17);

  raw_states[4] = !digitalRead(18);
  raw_states[5] = !digitalRead(19);
  raw_states[6] = !(analogRead(20)>400);
  raw_states[7] = !(analogRead(21)>400);
#ifdef LOG_TO_SERIAL
  for (int i=0;i<8;i++){
    Serial.print(" "); Serial.print(raw_states[i]); Serial.print(" ");
  }
  Serial.println();
#endif
}

#ifdef SEND_485
void send(int btn, long duration){
  String s = "{nano:"+String(ID)+",btn:"+String(btn)+",dura:"+String(duration)+"}";
  for (int i = 0; i<10; i++){
    if (send485(s.c_str())) break;
    Serial.println("collision detected, trying again:");
  }
}
#endif

void loop(){
  unsigned long now = micros();
  boolean pause = true;
  for (int i=0;i<8;i++){
    // we emulate a capacitor:
    // when the button is pressed, it slowly loads, if the button is released, it unloads.
    capacitor[i] = raw_states[i] ? capacitor[i]+1 : capacitor[i]>>1;

#ifdef DEBUG
    if (capacitor[i]) {
      Serial.print("cap "); Serial.print(i); Serial.print(": "); Serial.println(capacitor[i]);
    }
#endif    

    // if the capacitor reaches a certain treshold, the state is changed
    boolean new_state = capacitor[i]>TRESHOLD;

    // TODO: drop state after a maximum HIGH time

    // when the state changes, the duration of the last state is calculated
    if (states[i] != new_state){      
      long diff = now - times[i];
      times[i] = now;
#ifdef SEND_485
      if (states[i]){ // old state was HIGH → we are going LOW
        send(i+1,diff);
      }
#endif      
      states[i] = new_state;
    }
  }
}
